@@ -12,6 +12,7 @@ from PIL import Image
 from django.core.files.base import ContentFile
 from django.db.models import ImageField
 from django.db.models.fields.files import ImageFieldFile
+from django.db.models.signals import pre_delete
 
 from .widgets import ExtendedImageWidget
 from .utils import gen_filename
@@ -38,9 +39,9 @@ class ExtendedImageFieldFile(ImageFieldFile):
         super(ExtendedImageFieldFile, self).save(name, content, save)
         self.add_previews(name, content)
 
-    def delete(self, save=True):
-        self.delete_previews(self.name)
-        super(ExtendedImageFieldFile, self).delete(save)
+    def delete_file(self):
+        if self.storage.exists(self.path):
+            self.storage.delete(self.path)
 
     def add_previews(self, name, content):
         ext = os.path.splitext(name)[-1]
@@ -68,7 +69,8 @@ class ExtendedImageFieldFile(ImageFieldFile):
             preview_file.save(out, ext.lstrip('.').lower().replace('jpg', 'jpeg'))
             self.storage.save(preview_path, ContentFile(out.getvalue()))
 
-    def delete_previews(self, name):
+    def delete_previews(self, name=None):
+        name = name or self.name
         ext = os.path.splitext(name)[-1]
         image_name = os.path.basename(name).replace(ext, '')
 
@@ -126,16 +128,28 @@ class ExtendedImageField(ImageField):
     '''
     attr_class = ExtendedImageFieldFile
 
-    def __init__(self, previews=None, dir_sort=True, widget=None, widget_preview=None, **kwargs):
+    def __init__(self, previews=None, dir_sort=True, widget=None, widget_preview=None,
+                 cleanup=True, **kwargs):
         self.previews = previews
         self.dir_sort = dir_sort
         self.widget = widget
+        self.cleanup = cleanup
 
         if self.widget is None:
             preview = widget_preview or 'self'
             self.widget = ExtendedImageWidget(attrs={'preview': preview})
         super(ExtendedImageField, self).__init__(**kwargs)
 
+    def contribute_to_class(self, cls, name):
+        super(ExtendedImageField, self).contribute_to_class(cls, name)
+        pre_delete.connect(self.pre_delete_signal_handler, sender=cls)
+
     def formfield(self, **kwargs):
         kwargs['widget'] = self.widget
         return super(ExtendedImageField, self).formfield(**kwargs)
+
+    def pre_delete_signal_handler(self, *args, **kwargs):
+        if self.cleanup:
+            image = getattr(kwargs['instance'], self.attname)
+            image.delete_previews()
+            image.delete_file()
